@@ -50,6 +50,8 @@ extern cvar_t gl_zfix; // QuakeSpasm z-fighting fix
 
 extern gltexture_t *playertextures[MAX_SCOREBOARD]; //johnfitz
 
+cvar_t	r_lightmapwide = {"r_lightmapwide","0",CVAR_ROM};
+
 
 /*
 ====================
@@ -163,8 +165,6 @@ R_Init
 */
 void R_Init (void)
 {
-	extern cvar_t gl_finish;
-
 	Cmd_AddCommand ("timerefresh", R_TimeRefresh_f);
 	Cmd_AddCommand ("pointfile", R_ReadPointFile_f);
 
@@ -219,6 +219,8 @@ void R_Init (void)
 	Cvar_RegisterVariable (&r_noshadow_list);
 	Cvar_SetCallback (&r_noshadow_list, R_Model_ExtraFlags_List_f);
 	//johnfitz
+	Cvar_RegisterVariable (&r_lightmapwide);
+	Cvar_SetROM ("r_lightmapwide", gl_packed_pixels ? "1" : "0");
 
 	Cvar_RegisterVariable (&gl_zfix); // QuakeSpasm z-fighting fix
 	Cvar_RegisterVariable (&r_lavaalpha);
@@ -250,8 +252,10 @@ void R_TranslatePlayerSkin (int playernum)
 
 	//FIXME: if gl_nocolors is on, then turned off, the textures may be out of sync with the scoreboard colors.
 	if (!gl_nocolors.value)
+	{
 		if (playertextures[playernum])
 			TexMgr_ReloadImage (playertextures[playernum], top, bottom);
+	}
 }
 
 /*
@@ -332,6 +336,7 @@ static void R_ParseWorldspawn (void)
 		return; // error
 	if (com_token[0] != '{')
 		return; // error
+
 	while (1)
 	{
 		data = COM_Parse(data);
@@ -345,7 +350,7 @@ static void R_ParseWorldspawn (void)
 			q_strlcpy(key, com_token, sizeof(key));
 		while (key[0] && key[strlen(key)-1] == ' ') // remove trailing spaces
 			key[strlen(key)-1] = 0;
-		data = COM_Parse(data);
+		data = COM_ParseEx(data, CPE_ALLOWTRUNC);
 		if (!data)
 			return; // error
 		q_strlcpy(value, com_token, sizeof(value));
@@ -552,7 +557,7 @@ GLuint GL_CreateProgram (const GLchar *vertSource, const GLchar *fragSource, int
 	}
 	else
 	{
-		if (gl_num_programs == (sizeof(gl_programs)/sizeof(GLuint)))
+		if (gl_num_programs == Q_COUNTOF(gl_programs))
 			Host_Error ("gl_programs overflow");
 
 		gl_programs[gl_num_programs] = program;
@@ -584,7 +589,7 @@ void R_DeleteShaders (void)
 	gl_num_programs = 0;
 }
 
-GLuint current_array_buffer, current_element_array_buffer;
+static GLuint current_array_buffer, current_element_array_buffer;
 
 /*
 ====================
@@ -628,7 +633,7 @@ This must be called if you do anything that could make the cached bindings
 invalid (e.g. manually binding, destroying the context).
 ====================
 */
-void GL_ClearBufferBindings ()
+void GL_ClearBufferBindings (void)
 {
 	if (!gl_vbo_able)
 		return;
