@@ -947,7 +947,7 @@ static void Host_Reconnect_f (void)
 		return;
 
 	SCR_BeginLoadingPlaque ();
-	cls.signon = 0;		// need new connection messages
+	CL_ClearSignons ();		// need new connection messages
 }
 
 /*
@@ -990,36 +990,36 @@ Host_SavegameComment
 Writes a SAVEGAME_COMMENT_LENGTH character comment describing the current
 ===============
 */
-static void Host_SavegameComment (char *text)
+static void Host_SavegameComment (char text[SAVEGAME_COMMENT_LENGTH + 1])
 {
 	int		i;
 	char	kills[20];
-	char	*p1, *p2;
+	char	*p;
 
 	for (i = 0; i < SAVEGAME_COMMENT_LENGTH; i++)
 		text[i] = ' ';
-
-// Remove CR/LFs from level name to avoid broken saves, e.g. with autumn_sp map:
-// https://celephais.net/board/view_thread.php?id=60452&start=3666
-	p1 = strchr(cl.levelname, '\n');
-	p2 = strchr(cl.levelname, '\r');
-	if (p1 != NULL) *p1 = 0;
-	if (p2 != NULL) *p2 = 0;
+	text[SAVEGAME_COMMENT_LENGTH] = '\0';
 
 	i = (int) strlen(cl.levelname);
 	if (i > 22) i = 22;
 	memcpy (text, cl.levelname, (size_t)i);
+
+// Remove CR/LFs from level name to avoid broken saves, e.g. with autumn_sp map:
+// https://celephais.net/board/view_thread.php?id=60452&start=3666
+	while ((p = strchr(text, '\n')) != NULL)
+		*p = ' ';
+	while ((p = strchr(text, '\r')) != NULL)
+		*p = ' ';
+
 	sprintf (kills,"kills:%3i/%3i", cl.stats[STAT_MONSTERS], cl.stats[STAT_TOTALMONSTERS]);
 	memcpy (text+22, kills, strlen(kills));
+
 // convert space to _ to make stdio happy
 	for (i = 0; i < SAVEGAME_COMMENT_LENGTH; i++)
 	{
 		if (text[i] == ' ')
 			text[i] = '_';
 	}
-	if (p1 != NULL) *p1 = '\n';
-	if (p2 != NULL) *p2 = '\r';
-	text[SAVEGAME_COMMENT_LENGTH] = '\0';
 }
 
 /*
@@ -1251,6 +1251,10 @@ static void Host_Loadgame_f (void)
 		entnum++;
 	}
 
+	// Free edicts allocated during map loading but no longer used after restoring saved game state
+	for (i = entnum; i < sv.num_edicts; i++)
+		ED_Free(EDICT_NUM(i));
+
 	sv.num_edicts = entnum;
 	sv.time = time;
 
@@ -1265,6 +1269,9 @@ static void Host_Loadgame_f (void)
 		CL_EstablishConnection ("local");
 		Host_Reconnect_f ();
 	}
+
+	if (cls.state != ca_dedicated)
+		IN_Activate(); // moved to here from M_Load_Key()
 }
 
 //============================================================================
@@ -1601,10 +1608,8 @@ static void Host_PreSpawn_f (void)
 		return;
 	}
 
-	SZ_Write (&host_client->message, sv.signon.data, sv.signon.cursize);
-	MSG_WriteByte (&host_client->message, svc_signonnum);
-	MSG_WriteByte (&host_client->message, 2);
-	host_client->sendsignon = true;
+	host_client->sendsignon = PRESPAWN_SIGNONBUFS;
+	host_client->signonidx = 0;
 }
 
 /*
@@ -1723,7 +1728,7 @@ static void Host_Spawn_f (void)
 
 	MSG_WriteByte (&host_client->message, svc_signonnum);
 	MSG_WriteByte (&host_client->message, 3);
-	host_client->sendsignon = true;
+	host_client->sendsignon = PRESPAWN_FLUSH;
 }
 
 /*
