@@ -436,10 +436,14 @@ void M_ScanSaves (void)
 		loadable[i] = false;
 		q_snprintf (name, sizeof(name), "%s/s%i.sav", com_gamedir, i);
 		f = fopen (name, "r");
-		if (!f)
+		if (!f) {
 			continue;
-		fscanf (f, "%i\n", &version);
-		fscanf (f, "%79s\n", name);
+		}
+		if (fscanf(f, "%i\n", &version) != 1 ||
+		    fscanf(f, "%79s\n", name)   != 1) {
+			fclose(f);
+			continue;
+		}
 		q_strlcpy (m_filenames[i], name, SAVEGAME_COMMENT_LENGTH+1);
 
 	// change _ back to space
@@ -529,7 +533,6 @@ void M_Load_Key (int k)
 		if (!loadable[load_cursor])
 			return;
 		m_state = m_none;
-		IN_Activate();
 		key_dest = key_game;
 
 	// Host_Loadgame_f can't bring up the loading plaque because too much
@@ -1356,7 +1359,7 @@ const char *bindnames[][2] =
 	{"+movedown",		"swim down"}
 };
 
-#define	NUMCOMMANDS	(sizeof(bindnames)/sizeof(bindnames[0]))
+#define	NUMCOMMANDS	Q_COUNTOF(bindnames)
 
 static int	keys_cursor;
 static qboolean	bind_grab;
@@ -2541,7 +2544,6 @@ void M_Menu_Credits_f (void)
 //=============================================================================
 /* Menu Subsystem */
 
-
 void M_Init (void)
 {
 	Cmd_AddCommand ("togglemenu", M_ToggleMenu_f);
