@@ -55,6 +55,8 @@ int		type_size[8] = {
 	1	// sizeof(void *) / 4		// ev_pointer
 };
 
+#define NUM_TYPE_SIZES (int)Q_COUNTOF(type_size)
+
 static ddef_t	*ED_FieldAtOfs (int ofs);
 static qboolean	ED_ParseEpair (void *base, ddef_t *key, const char *s);
 
@@ -493,6 +495,9 @@ void ED_Print (edict_t *ed)
 	// if the value is still all 0, skip the field
 		type = d->type & ~DEF_SAVEGLOBAL;
 
+		if (type >= NUM_TYPE_SIZES)
+			continue;
+
 		for (j = 0; j < type_size[type]; j++)
 		{
 			if (v[j])
@@ -544,6 +549,10 @@ void ED_Write (FILE *f, edict_t *ed)
 
 	// if the value is still all 0, skip the field
 		type = d->type & ~DEF_SAVEGLOBAL;
+
+		if (type >= NUM_TYPE_SIZES)
+			continue;
+
 		for (j = 0; j < type_size[type]; j++)
 		{
 			if (v[j])
@@ -910,7 +919,10 @@ const char *ED_ParseEdict (const char *data, edict_t *ent)
 		}
 
 		// parse value
-		data = COM_Parse (data);
+		// HACK: we allow truncation when reading the wad field,
+		// otherwise maps using lots of wads with absolute paths
+		// could cause a parse error
+		data = COM_ParseEx (data, !strcmp (keyname, "wad") ? CPE_ALLOWTRUNC : CPE_NOTRUNC);
 		if (!data)
 			Host_Error ("ED_ParseEntity: EOF without closing brace");
 
@@ -1036,6 +1048,8 @@ void ED_LoadFromFile (const char *data)
 			continue;
 		}
 
+		SV_ReserveSignonSpace (512);
+
 		pr_global_struct->self = EDICT_TO_PROG(ent);
 		PR_ExecuteProgram (func - pr_functions);
 	}
