@@ -36,7 +36,7 @@ byte *SV_FatPVS (vec3_t org, qmodel_t *worldmodel);
 
 /*
 ================
-R_ClearTextureChains -- ericw 
+R_ClearTextureChains -- ericw
 
 clears texture chains for all textures used by the given model, and also
 clears the lightmap chains
@@ -353,7 +353,7 @@ static unsigned int num_vbo_indices;
 R_ClearBatch
 ================
 */
-static void R_ClearBatch ()
+static void R_ClearBatch (void)
 {
 	num_vbo_indices = 0;
 }
@@ -365,7 +365,7 @@ R_FlushBatch
 Draw the current batch if non-empty and clears it, ready for more R_BatchSurface calls.
 ================
 */
-static void R_FlushBatch ()
+static void R_FlushBatch (void)
 {
 	if (num_vbo_indices > 0)
 	{
@@ -527,7 +527,7 @@ void R_DrawTextureChains_TextureOnly (qmodel_t *model, entity_t *ent, texchain_t
 /*
 ================
 GL_WaterAlphaForEntitySurface -- ericw
- 
+
 Returns the water alpha to use for the entity and surface combination.
 ================
 */
@@ -547,13 +547,15 @@ extern GLuint gl_bmodel_vbo;
 // uniforms used in vert shader
 
 // uniforms used in frag shader
-static GLuint texLoc;
-static GLuint LMTexLoc;
-static GLuint fullbrightTexLoc;
-static GLuint useFullbrightTexLoc;
-static GLuint useOverbrightLoc;
-static GLuint useAlphaTestLoc;
-static GLuint alphaLoc;
+static GLint  texLoc;
+static GLint  LMTexLoc;
+static GLint  fullbrightTexLoc;
+static GLint  useFullbrightTexLoc;
+static GLint  useOverbrightLoc;
+static GLint  useAlphaTestLoc;
+static GLint  useLightmapWideLoc;
+static GLint  useLightmapOnlyLoc;
+static GLint  alphaLoc;
 
 #define vertAttrIndex 0
 #define texCoordsAttrIndex 1
@@ -611,6 +613,9 @@ void R_DrawTextureChains_Water (qmodel_t *model, entity_t *ent, texchain_t chain
 	}
 	else if (cl.worldmodel->haslitwater && r_litwater.value && r_world_program != 0)
 	{
+		const int overbright = !!gl_overbright.value;
+		const int wide10bits = !!r_lightmapwide.value;
+
 		has_lit_water = true;
 
 		GL_UseProgramFunc (r_world_program);
@@ -632,8 +637,10 @@ void R_DrawTextureChains_Water (qmodel_t *model, entity_t *ent, texchain_t chain
 		GL_Uniform1iFunc (LMTexLoc, 1);
 		GL_Uniform1iFunc (fullbrightTexLoc, 2);
 		GL_Uniform1iFunc (useFullbrightTexLoc, 0);
-		GL_Uniform1iFunc (useOverbrightLoc, (int)gl_overbright.value);
+		GL_Uniform1iFunc (useOverbrightLoc, overbright);
 		GL_Uniform1iFunc (useAlphaTestLoc, 0);
+		GL_Uniform1iFunc (useLightmapWideLoc, wide10bits);
+		GL_Uniform1iFunc (useLightmapOnlyLoc, 0);
 
 		for (i=0 ; i<model->numtextures ; i++)
 		{
@@ -700,7 +707,7 @@ void R_DrawTextureChains_Water (qmodel_t *model, entity_t *ent, texchain_t chain
 	}
 	else
 		has_unlit_water = true;
-	
+
 	if (has_unlit_water)
 	{
 		// Unlit water
@@ -843,6 +850,8 @@ void GLWorld_CreateShaders (void)
 		"uniform bool UseFullbrightTex;\n"
 		"uniform bool UseOverbright;\n"
 		"uniform bool UseAlphaTest;\n"
+		"uniform bool UseLightmapWide;\n"
+		"uniform bool UseLightmapOnly;\n"
 		"uniform float Alpha;\n"
 		"\n"
 		"varying float FogFragCoord;\n"
@@ -850,9 +859,13 @@ void GLWorld_CreateShaders (void)
 		"void main()\n"
 		"{\n"
 		"	vec4 result = texture2D(Tex, gl_TexCoord[0].xy);\n"
+		"	if (UseLightmapOnly)\n"
+		"		result = vec4(0.5, 0.5, 0.5, 1.0);\n"
 		"	if (UseAlphaTest && (result.a < 0.666))\n"
 		"		discard;\n"
 		"	result *= texture2D(LMTex, gl_TexCoord[1].xy);\n"
+		"	if (UseLightmapWide)\n"
+		"	    result.rgb *= 4.0;\n"
 		"	if (UseOverbright)\n"
 		"		result.rgb *= 2.0;\n"
 		"	if (UseFullbrightTex)\n"
@@ -868,7 +881,7 @@ void GLWorld_CreateShaders (void)
 	if (!gl_glsl_alias_able)
 		return;
 
-	r_world_program = GL_CreateProgram (vertSource, fragSource, sizeof(bindings)/sizeof(bindings[0]), bindings);
+	r_world_program = GL_CreateProgram (vertSource, fragSource, Q_COUNTOF(bindings), bindings);
 	
 	if (r_world_program != 0)
 	{
@@ -879,6 +892,8 @@ void GLWorld_CreateShaders (void)
 		useFullbrightTexLoc = GL_GetUniformLocation (&r_world_program, "UseFullbrightTex");
 		useOverbrightLoc = GL_GetUniformLocation (&r_world_program, "UseOverbright");
 		useAlphaTestLoc = GL_GetUniformLocation (&r_world_program, "UseAlphaTest");
+		useLightmapWideLoc = GL_GetUniformLocation (&r_world_program, "UseLightmapWide");
+		useLightmapOnlyLoc = GL_GetUniformLocation (&r_world_program, "UseLightmapOnly");
 		alphaLoc = GL_GetUniformLocation (&r_world_program, "Alpha");
 	}
 }
@@ -893,15 +908,17 @@ Requires 3 TMUs, OpenGL 2.0
 */
 void R_DrawTextureChains_GLSL (qmodel_t *model, entity_t *ent, texchain_t chain)
 {
+	const float	entalpha = (ent != NULL) ?
+			 ENTALPHA_DECODE(ent->alpha) : 1.0f;
+	const int	overbright = !!gl_overbright.value;
+	const int	wide10bits = !!r_lightmapwide.value;
+
 	int			i;
 	msurface_t	*s;
 	texture_t	*t;
 	qboolean	bound;
 	int		lastlightmap;
 	gltexture_t	*fullbright = NULL;
-	float		entalpha;
-
-	entalpha = (ent != NULL) ? ENTALPHA_DECODE(ent->alpha) : 1.0f;
 
 // enable blending / disable depth writes
 	if (entalpha < 1)
@@ -929,8 +946,10 @@ void R_DrawTextureChains_GLSL (qmodel_t *model, entity_t *ent, texchain_t chain)
 	GL_Uniform1iFunc (LMTexLoc, 1);
 	GL_Uniform1iFunc (fullbrightTexLoc, 2);
 	GL_Uniform1iFunc (useFullbrightTexLoc, 0);
-	GL_Uniform1iFunc (useOverbrightLoc, (int)gl_overbright.value);
+	GL_Uniform1iFunc (useOverbrightLoc, overbright);
 	GL_Uniform1iFunc (useAlphaTestLoc, 0);
+	GL_Uniform1iFunc (useLightmapWideLoc, wide10bits);
+	GL_Uniform1iFunc (useLightmapOnlyLoc, 0);
 	GL_Uniform1fFunc (alphaLoc, entalpha);
 
 	for (i=0 ; i<model->numtextures ; i++)
@@ -1000,6 +1019,90 @@ void R_DrawTextureChains_GLSL (qmodel_t *model, entity_t *ent, texchain_t chain)
 	}
 }
 
+/*
+================
+R_DrawLightmapChains_GLSL -- ericw
+================
+*/
+void R_DrawLightmapChains_GLSL(qmodel_t* model, entity_t* ent, texchain_t chain)
+{
+	const int	overbright = !!gl_overbright.value;
+	const int	wide10bits = !!r_lightmapwide.value;
+
+	int			i;
+	msurface_t* s;
+	texture_t* t;
+	int		lastlightmap;
+
+	GL_UseProgramFunc(r_world_program);
+
+	// Bind the buffers
+	GL_BindBuffer(GL_ARRAY_BUFFER, gl_bmodel_vbo);
+	GL_BindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0); // indices come from client memory!
+
+	GL_EnableVertexAttribArrayFunc(vertAttrIndex);
+	GL_EnableVertexAttribArrayFunc(texCoordsAttrIndex);
+	GL_EnableVertexAttribArrayFunc(LMCoordsAttrIndex);
+
+	GL_VertexAttribPointerFunc(vertAttrIndex, 3, GL_FLOAT, GL_FALSE, VERTEXSIZE * sizeof(float), ((float*)0));
+	GL_VertexAttribPointerFunc(texCoordsAttrIndex, 2, GL_FLOAT, GL_FALSE, VERTEXSIZE * sizeof(float), ((float*)0) + 3);
+	GL_VertexAttribPointerFunc(LMCoordsAttrIndex, 2, GL_FLOAT, GL_FALSE, VERTEXSIZE * sizeof(float), ((float*)0) + 5);
+
+	// set uniforms
+	GL_Uniform1iFunc(texLoc, 0);
+	GL_Uniform1iFunc(LMTexLoc, 1);
+	GL_Uniform1iFunc(fullbrightTexLoc, 2);
+	GL_Uniform1iFunc(useFullbrightTexLoc, 0);
+	GL_Uniform1iFunc(useOverbrightLoc, overbright);
+	GL_Uniform1iFunc(useAlphaTestLoc, 0);
+	GL_Uniform1iFunc(useLightmapWideLoc, wide10bits);
+	GL_Uniform1fFunc(alphaLoc, 1.0f);
+	GL_Uniform1iFunc(useFullbrightTexLoc, 0);
+	GL_Uniform1iFunc(useLightmapOnlyLoc, 1);
+
+	R_ClearBatch();
+	lastlightmap = -1;
+
+	for (i = 0; i < model->numtextures; i++)
+	{
+		t = model->textures[i];
+
+		if (!t || !t->texturechains[chain] || t->texturechains[chain]->flags & (SURF_DRAWTILED | SURF_NOTEXTURE))
+			continue;
+
+		if (t->texturechains[chain]->texinfo->flags & TEX_SPECIAL)
+			continue; // unlit water
+
+		for (s = t->texturechains[chain]; s; s = s->texturechain)
+		{
+			if (s->lightmaptexturenum < 0)
+				continue;
+
+			if (s->lightmaptexturenum != lastlightmap)
+			{
+				R_FlushBatch();
+
+				GL_SelectTexture(GL_TEXTURE1);
+				GL_Bind(lightmaps[s->lightmaptexturenum].texture);
+				lastlightmap = s->lightmaptexturenum;
+			}
+			R_BatchSurface(s);
+
+			rs_brushpasses++;
+		}
+	}
+
+	R_FlushBatch();
+
+	// clean up
+	GL_DisableVertexAttribArrayFunc(vertAttrIndex);
+	GL_DisableVertexAttribArrayFunc(texCoordsAttrIndex);
+	GL_DisableVertexAttribArrayFunc(LMCoordsAttrIndex);
+
+	GL_UseProgramFunc(0);
+	GL_SelectTexture(GL_TEXTURE0);
+}
+
 /*
 =============
 R_DrawWorld -- johnfitz -- rewritten
@@ -1034,6 +1137,12 @@ void R_DrawTextureChains (qmodel_t *model, entity_t *ent, texchain_t chain)
 
 	if (r_lightmap_cheatsafe)
 	{
+		if (r_world_program != 0)
+		{
+			R_DrawLightmapChains_GLSL(model, ent, chain);
+			return;
+		}
+
 		if (!gl_overbright.value)
 		{
 			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
